#include <assert.h>

#include "damsel.h"
#include "damsel-internal.h"

/** the free_entry parameter should free the supplied memory. it
    should also free any other internal bits that are
    type specific. if nothing interesting is going on there, then
    the regular libc 'free' can be used for the value of this
    parameter. */
void linkedlist_free(linkedlist *ll, void (*free_entry)(void *m) ) {
  if(ll == NULL) return;
  if(free_entry != NULL) free_entry(ll->head);
  linkedlist_free(ll->next, free_entry);
  free(ll);
}


void linkedlist_prepend(linkedlist **la, void *entry) {
  assert(la!=NULL);
  linkedlist *ll = malloc(sizeof(linkedlist));
  assert(ll!=NULL);
  ll->head = entry;
  ll->next = *la;
  *la = ll;
}

void linkedlist_append(linkedlist **la, void *entry) {
  linkedlist *ll = malloc(sizeof(linkedlist));
  ll->head = entry;

  while(*la != NULL) {
    la = &( (*la)->next );
  }

  assert(*la == NULL);
  *la = ll;
  ll->next = NULL;
  return;
}

int linkedlist_length(linkedlist *l) {
  linkedlist *here = l;
  int c=0;
  while(here!=NULL) {
    c++;
    here=here->next;
  }
  return c;
}

linkedlist *linkedlist_filter(linkedlist *list, int (*filter)(void *d)) {
  linkedlist *output = NULL;
  linkedlist *input;
  for(input = list; input != NULL ; input = input->next) {
    if((*filter)(input->head)) {
      linkedlist *newhead = malloc(sizeof(linkedlist));
      assert(newhead != NULL);
      newhead->next = output;
      newhead->head = input->head;
      output = newhead;
    }
  }
  return output;
}

