#include "damsel.h"

#include <string.h>
#include <assert.h>

#include "damsel-sugar.h"
#include "damsel-internal.h"

damsel_model DMSLmodel_attached_with_trait(MPI_Comm comm, char *fname, char * mode) {
    damsel_model model;
    damsel_trait_bundle tb;
    damsel_err_t err;

    assert(strncmp(mode, "READ", strlen("READ"))
	    || strncmp(mode, "WRITE", strlen("WRITE"))  );

    err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);

    err = DMSLtrait_create(&tb);
    err = DMSLtrait_put(tb, "mode", mode);
    
    DMSL_CHECK(DMSLmodel_attach(model, fname, comm, tb));
    err = DMSLtrait_release(tb);
    return model;
}

int64_t DMSL_subprod(int ndims, int64_t *dims) {
    if (ndims == 1) {
	return dims[0];
    } 
    return dims[0] * DMSL_subprod(ndims-1, dims+1);
}

    /* some notes on sub-selections:
     * - DMSLselect_range will not do what you think it might.  It inspects the
     *   data corresponding to "tag_handle" and returns, in the container,
     *   handles that fall into that range. 
     * - DMSLselect_handles_with_values will return all handles. 
     * - we have this point-wise selection routine. It's the best we can do at
     *   this time */
static int select_handles(damsel_container container, damsel_handle *selection, 
	int ndims, int64_t *size, int64_t *start, int64_t *count, int64_t offset) {

#if 0
    /* the non-recursive, explicitly 2-d approach */
    selection_handles[i*count[1]+jk] =
	DMSLcontainer_handle_at_position(full_container,
		(i+start[0])*size[1] + start[1]+jk);

#endif

    int i, j; 
    int64_t nitems, stride;
    if (ndims == 1) {
	assert(start[0] + count[0] <= size[0]);
	for(i=0, j=start[0]; i< count[0]; j++, i++) {
	    selection[i] = 
		DMSLcontainer_handle_at_position(container, offset+j);
	}
	return (count[0]);
    }

    stride = DMSL_subprod(ndims-1, size+1);
    /* tracking three differnt things in this loop:
     * - how many items we selected (not really used for much now but might be useful later)
     * - number of times we've iterated (from 0 to 'count' times)
     * - where we are in the array (the 'start' array) */
    for (nitems=0, j=0, i=start[0]; j<count[0]; j++, i++) {
	nitems += select_handles(container, selection+nitems, 
		ndims-1, size+1, start+1, count+1, offset + stride*i);
    }
    return nitems;
}


/* given a subarray tagged on a collection of handles (the tag), return a
 * desired subset of handles.  Requires, by convention, prior knowledge of the
 * layout of the handles (otherwise any N-d array looks like 1d */

damsel_container DMSLselect_subset_from_tag(damsel_model model, 
	damsel_handle tag_id, int ndims, int64_t * size, int64_t *start, 
	int64_t *count)
{
    int nitems, ret;
    damsel_container full_container, selection_container;
    damsel_model_internal *model_internal;
    damsel_model file_model;
    damsel_handle *selection_handles;
    damsel_err_t err;

    model_internal = (damsel_model_internal*)model;
    file_model = (damsel_model) model_internal->attached_file;

    nitems = DMSL_subprod(ndims, count);
    selection_handles = calloc(nitems, sizeof(damsel_handle));

    full_container = DMSLselect_handles_with_values(model, tag_id);

    ret = select_handles(full_container, selection_handles, ndims, 
	    size, start, count, 0);
    assert(nitems == ret);


    err = DMSLcontainer_create_vector(file_model, selection_handles,
	    nitems, &selection_container);
    DMSL_CHECK(DMSLcontainer_release(full_container));
    free(selection_handles);
    
    return(selection_container);
}

