/* this is for implementation-internal datatype definitions that are not exposed
through the public API */

void release_types(void);
#define DAMSEL_I_TYPE_ARRAY_MAX_NDIMS       32
#define DAMSEL_I_TYPE_MAX_NAME_LEN          80
#define DAMSEL_I_TYPE_MAX_NMEMBS            32
#define DAMSEL_I_TYPE_MAX_VECTOR_SIZE 		256

#define DAMSEL_I_TYPE_OPAQUE_MAGIC 0x34281
typedef struct {
  int magic;                            /* Must be first for type-mapping code */
  size_t bytecount;
} damsel_i_type_opaque;

#define DAMSEL_I_TYPE_ARRAY_MAGIC 0x34291
typedef struct {
  int magic;                                    /* Must be first for type-mapping code */
  damsel_data_type base_type;                   /* Base type of the array (i.e. an array of <base_type>) */
  unsigned ndims;                               /* Number of dimensions of array */
  size_t dims[DAMSEL_I_TYPE_ARRAY_MAX_NDIMS];   /* Sizes of array dimensions */
} damsel_i_type_array;

/* A vector datatype */
#define DAMSEL_I_TYPE_VECTOR_MAGIC 0x34301
typedef struct {
  int magic;                    /* Must be first for type-mapping code */
  damsel_data_type base_type;   /* Base type of each element */
  size_t max_len;               /* HDF5 parallel IO does not support writing variable-length
                                   data. For a temporary solution, we will use fixed length
                                   array type of size max_len. */								   
} damsel_i_type_vector;

/* A enum datatype */
#define DAMSEL_I_TYPE_ENUM_MAGIC 0x34311
typedef struct {
  int      magic;                            /* Must be first for type-mapping code */
  unsigned nmembs;                           /* number of members defined in enum   */
  char     names[DAMSEL_I_TYPE_MAX_NMEMBS][DAMSEL_I_TYPE_MAX_NAME_LEN];   /* names  */
  int      values[DAMSEL_I_TYPE_MAX_NMEMBS]; /* values of enum fields               */
} damsel_i_type_enum;

/* A struct datatype */
#define DAMSEL_I_TYPE_STRUCT_MAGIC 0x34321
typedef struct {
  int              magic;                              /* Must be first for type-mapping code */
  unsigned         nmembs;                             /* number of members defined in struct */
  char             names[DAMSEL_I_TYPE_MAX_NMEMBS][DAMSEL_I_TYPE_MAX_NAME_LEN]; /* names      */
  damsel_data_type types[DAMSEL_I_TYPE_MAX_NMEMBS];    /* types of the struct members         */ 
  size_t           offsets[DAMSEL_I_TYPE_MAX_NMEMBS];
  size_t           tsize;
} damsel_i_type_struct;



