#ifdef DAMSEL_HDF5_MULTI
#ifndef HAVE_H5DWRITE_MULTI
# include "h5multidataset.h"

typedef struct H5F_t H5F_t;							  

herr_t H5Dclose_multi(size_t count, H5D_rw_multi_t info[])
{
    int i;
    herr_t ret_val = 0;
	
	if (count<=0)
	    return 0;

	for (i=0; i<count; i++) {
	    H5Tclose (info[i].mem_type_id);
	    H5Sclose (info[i].mem_space_id);
	    H5Sclose (info[i].file_space_id);
	    ret_val = H5Dclose(info[i].dset_id);
	    if (info[i].rbuf != NULL) free (info[i].rbuf);
	    if (info[i].wbuf != NULL) free ((void*) info[i].wbuf);
	}
	
	return ret_val;
}

herr_t H5Dread_multi(hid_t file_id,  size_t count,
                    H5D_rw_multi_t info[],
                    hid_t xfer_plist_id)
{
    int i;
    herr_t ret_val = 0;
	
	if (count<=0 || info==NULL)
	    return 0;

	for (i=0; i<count; i++) {
	    ret_val = H5Dread(info[i].dset_id, info[i].mem_type_id, 
		info[i].mem_space_id, info[i].file_space_id, xfer_plist_id, 
		info[i].rbuf); 
		if (ret_val < 0)
		    return ret_val;
	}
	
	return ret_val;
}

herr_t H5Dwrite_multi(hid_t file_id, size_t count,
                    H5D_rw_multi_t info[],
                    hid_t xfer_plist_id)
{
    int i;
    herr_t ret_val = 0;

	hid_t fapl = H5Fget_access_plist(file_id);
	if (fapl>0) {
	    hid_t driver = H5Pget_driver(fapl);
	    H5Pclose(fapl);
	
		if (H5FD_MPIO==driver || H5FD_MPIPOSIX==driver) {
			H5FD_mpio_xfer_t xfer_mode;
			H5Pget_dxpl_mpio(xfer_plist_id, &xfer_mode);
			
			if (H5FD_MPIO_COLLECTIVE==xfer_mode) {
				H5F_t *file = (H5F_t *)H5I_object_verify(file_id, H5I_FILE);
				if (file) {
					MPI_Comm comm = H5F_mpi_get_comm(file);
					if (MPI_COMM_NULL != comm)
						MPI_Barrier(comm);
				}
				
				/* for now, we are going to switch to independent IO. The real
				   implementation will take care of the collective IO case: wirte
				   zero byte to the file
				*/
				H5Pset_dxpl_mpio (xfer_plist_id, H5FD_MPIO_INDEPENDENT);	
			}
		} /* (H5FD_MPIO==driver || H5FD_MPIPOSIX==driver) */	
	}
		
	if (count<=0 || info==NULL)
	    return 0;

	for (i=0; i<count; i++) {
        ret_val = H5Dwrite(info[i].dset_id, info[i].mem_type_id, 
		info[i].mem_space_id, info[i].file_space_id, xfer_plist_id, 
		info[i].wbuf); 
		
		if (ret_val < 0)
		    return ret_val;
	}
	
	return ret_val;
}							  
#endif
#endif
