
#include <assert.h>
#include <stdio.h>
#include <unistd.h>

#include "damsel.h"

#define FILENAME "output/test-connectivity-write-pi-ord.h5"

int main(int argc, char **argv) {
#ifndef DAMSEL_SINGLETHREAD
  int res;
#endif
  printf("%s\n", argv[0]);

#ifndef DAMSEL_SINGLETHREAD
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  unlink(FILENAME);
  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "WRITE");

  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);

  damsel_handle edge_vector[5] = {100, 101, 102, 103, 104};
  damsel_container edge_handle_collect_id;
  err = DMSLcontainer_create_vector(model, edge_vector, /* count = */ 5, &edge_handle_collect_id);

  DMSLmodel_map_handles_inventing_file_handles(edge_handle_collect_id);


  // we need to map the handles for the points that edges connect, separately from
  // declaring the connectivity.

  damsel_container point_container_id;
  err = DMSLcontainer_create_sequence(model, 10, 6, 1, &point_container_id);

// TODO: this needs to happen in a way that rearranges the order
// of things in the file.

// the present call:
//    allocates file side handles
//    creates storage on disk
//    creates a mapping from the supplied point_container to the
//                newly allocated space.

// that together causes the data to be written out to disk in the same
// order as it is supplied in point_container.

// but I'd like to put in a permutation somewhere. I'm not sure what the
// mechanism for that should be.

// I need to:
//   get the right number of file side handles.
//   make space for those file side handles in the file.
//   map those file side handles to memory size handles, but in a
//          different order.

// so where's the right place to do that permutation?
// at the moment, I assume that there's a local rank app-side handle
// for everything that I'm mapping. scalabilitywise, that may not
// be true in the long term, but whatever for now.

// at the time of this call, I'm not going to be in a position to make
// file-side handles.

// so: I can either defer the permutation calculation till later?
//     which could be useful becuase then I don't need connectivity info
//     defined at the moment
// or: should i allow the specification of a permutation now, in more
//     abstract-from-file sense: eg a permutation of app side handles?
// the declare-then-execute model probably prefers that first one.
// a functional programming mindset says I should be putting in a
// function there. in C that would be a callback...
// so then what would the callback do?
// it would need to do the mapping in a more complicated way than
// the basic "map the container on disk". so perhaps there should
// initially be an "id" callback which really does map the container
// on disk; and then I want to make a connectivity-aware one, which
// maps differently.

// so then what does the 'layout' callback need?
// probably:
//   i) before hand, damsel has allocated the file side handles and
//      we can assume that they will be in order on disk, however
//      that happens.
//  ii) when it ends, the callback needs to have mapped all of the
//      appside handles into the file-side handles passed in.
//      ... but it is not required to map them in any particular order.

  DMSLmodel_map_handles_inventing_file_handles_with_permute(point_container_id, damsel_permute_connectivity);

  //  13 11 14 10 15 12
  //  3  1  4  1  5   9

  // 10 11 12 13 14 15
  // 1  1  9  3  4  5

  /* define handle collection for use with defining (edge) entity connectivity */
  damsel_handle edge_connect[10] = {13, 11, 11, 14, 14, 10, 10, 15, 15, 12};

  damsel_container edge_connect_handle_collect_id;
  err = DMSLcontainer_create_vector(model, edge_connect, /* count = */ 10, &edge_connect_handle_collect_id);

//  DMSLmodel_map_handles_inventing_file_handles(edge_connect_handle_collect_id); // BUG: this mapping should fail because it contains duplicate handles targetted at the same place.

  DMSLentity_define(edge_handle_collect_id, DAMSEL_ENTITY_TYPE_EDGE, 2,
    edge_connect_handle_collect_id);

  short int data[] = { 1,1,9,3,4,5 };

  damsel_handle data_tag_handle = 0x5000;

  DMSLtag_define(model, &data_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "pidigits");
  DMSLmodel_map_single_handle_inventing_file_handle(model, data_tag_handle);
  DMSLmodel_map_tag(data, point_container_id, &data_tag_handle);

  printf("test-connectivity-write-pi: edge_handle_collect_id = %p, edge_connect_handle_collect_id = %p\n", edge_handle_collect_id, edge_connect_handle_collect_id);

  DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_WRITE);
  DMSLmodel_close(model);
  DMSLcontainer_close(point_container_id);
  DMSLcontainer_close(edge_connect_handle_collect_id);
  DMSLcontainer_close(edge_handle_collect_id);

  DMSLlib_finalize(lib);

  MPI_Finalize();
  printf("done!\n");
  return 0;
}

