
#include <assert.h>
#include <stdio.h>
#include <unistd.h>

#include "damsel.h"

#define FILENAME "output/test-connectivity-write-pi-wrong.h5"

int main(int argc, char **argv) {
#ifndef DAMSEL_SINGLETHREAD  
  int res;
#endif
  printf("%s\n", argv[0]);

#ifndef DAMSEL_SINGLETHREAD
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  unlink(FILENAME);
  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "WRITE");

  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);

  damsel_handle edge_vector[5] = {100, 101, 102, 103, 104};
  damsel_container edge_handle_collect_id;
  err = DMSLcontainer_create_vector(model, edge_vector, /* count = */ 5, &edge_handle_collect_id);

  DMSLmodel_map_handles_inventing_file_handles(edge_handle_collect_id);


  // we need to map the handles for the points that edges connect, separately from
  // declaring the connectivity.

  damsel_container point_container_id;
  err = DMSLcontainer_create_sequence(model, 10, 6, 1, &point_container_id);

  DMSLmodel_map_handles_inventing_file_handles(point_container_id);

  //  13 11 14 10 15 12
  //  3  1  4  1  5   9

  // 10 11 12 13 14 15
  // 1  1  9  3  4  5

  /* define handle collection for use with defining (edge) entity connectivity */
  damsel_handle edge_connect[10] = {13, 11, 11, 10, 10, 14, 14, 15, 15, 12};

  damsel_container edge_connect_handle_collect_id;
  err = DMSLcontainer_create_vector(model, edge_connect, /* count = */ 10, &edge_connect_handle_collect_id);


//  DMSLmodel_map_handles_inventing_file_handles(edge_connect_handle_collect_id); // BUG: this mapping should fail because it contains duplicate handles targetted at the same place.

  DMSLentity_define(edge_handle_collect_id, DAMSEL_ENTITY_TYPE_EDGE, 2,
    edge_connect_handle_collect_id);

  short int data[] = { 1,1,9,3,4,5 };

  damsel_handle data_tag_handle = 0x5000;

  DMSLtag_define(model, &data_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "pidigits");
  DMSLmodel_map_single_handle_inventing_file_handle(model, data_tag_handle);
  DMSLmodel_map_tag(data, point_container_id, &data_tag_handle);

  printf("test-connectivity-write-pi: edge_handle_collect_id = %p, edge_connect_handle_collect_id = %p\n", edge_handle_collect_id, edge_connect_handle_collect_id);

  DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_WRITE);
  DMSLmodel_close(model);
  DMSLcontainer_close(edge_handle_collect_id);
  DMSLcontainer_close(edge_connect_handle_collect_id);
  DMSLcontainer_close(point_container_id);

  DMSLlib_finalize(lib);

  MPI_Finalize();
  printf("done!\n");
  return 0;
}

