#include "damsel.h"
#include "damsel-internal.h"

#include <unistd.h>
#include "damsel-sugar.h"

/* the test: two processes tag the same entity with the same value.  
 * expected result: If each process writes N identical items to the same place,
 *                  should get back a count of N items 
 * observed result (test-pnetcdf): count reported as 2N, not N */

#define FILENAME "output/test-container_count-parallel.dmsl"
#define TAGNAME "identifiers"
#define DATA_PER_PROC 4

void init_file(MPI_Comm comm)
{
    damsel_model model; 
    int i, data[DATA_PER_PROC], rank, nprocs;
    damsel_data values;
    int handle_counter = 800;
    int remote_handle, *remote_handle_vector;
    damsel_err_t err;

    MPI_Comm_rank(comm, &rank);
    MPI_Comm_size(comm, &nprocs);

    unlink(FILENAME);

    model = DMSLmodel_attached_with_trait(comm, FILENAME, "WRITE");

    for (i=0; i<DATA_PER_PROC; i++)
	data[i] = 100+i;

    remote_handle = handle_counter;

    err = DMSLcontainer_create_sequence(model, handle_counter++, 
	    DATA_PER_PROC, 1, &values.container);
    values.tag_handle = handle_counter += DATA_PER_PROC;
    
    DMSL_CHECK(DMSLtag_define(model, &(values.tag_handle), DAMSEL_DATA_TYPE_INTEGER,
		TAGNAME));
    DMSL_CHECK(DMSLmodel_map_tag(data, values.container, &(values.tag_handle)));

    remote_handle_vector = calloc(nprocs, sizeof(int));
    if (rank != 0) {
	int tagrank = 0;
	int *proc_ranks = calloc(nelems, sizeof(int)); /* an all-zero array
							  means all processors
							  look to rank zero for
							  official mapping */
	DMSL_CHECK(DMSLmodel_map_global_handles(values.container, proc_ranks, values.

    DMSL_CHECK(DMSLmodel_map_handles_inventing_file_handles(values.container));
    DMSL_CHECK(DMSLmodel_map_single_handle_inventing_file_handle(model, 
		values.tag_handle));

    DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_WRITE);

    DMSL_CHECK(DMSLcontainer_release(values.container));
    DMSL_CHECK(DMSLmodel_close(model));

    return;
}

int count_items(MPI_Comm comm)
{
    damsel_model model, fileside_model;
    damsel_model_internal *model_internal;
    damsel_data item;
    damsel_err_t err;

    model = DMSLmodel_attached_with_trait(comm, FILENAME, "READ");
    DMSL_CHECK(DMSLmodel_populate(model));

    model_internal = (damsel_model_internal*)model;
    fileside_model = (damsel_model) model_internal->attached_file;

    err = DMSLselect_tag_by_name(model, TAGNAME, &item.tag_handle);
    err = DMSLcontainer_create_sequence(fileside_model, 
	    item.tag_handle, 1, 1, &item.tag_container);

    item.container = DMSLselect_handles_with_values(model, item.tag_handle);
    return DMSLcontainer_count(item.container);

}

int main(int argc, char ** argv)
{
    int lvl;
    int nitems;

    MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &lvl);
    damsel_library lib;
    damsel_err_t err = DMSLlib_init(&lib);
    
    /* recent mailing list discussion focused on three ways to "count" things:
     * Q1: If one process writes out 4 values onto a tag, all equal to 1, how
     * many values will I get back?
     *
     * 4
     *
     * Q2: If one process writes out N*4 values onto a tag, all equal to 1, how
     * many values will I get back?
     *
     * N*4.
     *
     * Q3: If N processes each write out 4 values onto a tag, all four equal to
     * 1, how many values will I get back?
     *
     * N*4.
     *
     * TODO: update this test so that it exercises those three approaches.
     * Probably I will tag three different values */
    init_file(MPI_COMM_WORLD);

    nitems = count_items(MPI_COMM_WORLD);

    fprintf(stderr, "tag %s contains %d items.  Expected %d items\n",
	    TAGNAME, nitems, DATA_PER_PROC);

    DMSLlib_finalize(lib);
    MPI_Finalize();

    return 0;
}
