/* test create_vector and writing to disk */

#include <assert.h>
#include <stdio.h>
#include <mpi.h>
#include <unistd.h>

#include "damsel.h"

#define FILENAME "output/test-create-vector.h5"

int main(int argc, char **argv) {
#ifndef DAMSEL_SINGLETHREAD
  int res;
#endif
  short int data[] = {3, 1, 4, 1, 5}; // some arbitrary data to write

  printf("%s\n", argv[0]);

#ifndef DAMSEL_SINGLETHREAD
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);

  /* define a tag which will store our data. the tag will be
     named 'somedata' */

  damsel_handle data_tag_handle = 0x10000;
  DMSLtag_define(model, &data_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "somedata");

  /* we'll create a vector handle collection with 5 handles in it. */

  damsel_handle vector_of_handles[] = { 0x20000, 0x21000, 0x22000,
                                        0x23000, 0x24000 };
  damsel_container vertex_collection_id;
  err = DMSLcontainer_create_vector(model, vector_of_handles, 5, &vertex_collection_id);

  /* now set the values of the somedata tag on the handles */
  DMSLmodel_map_tag(data, vertex_collection_id, &data_tag_handle);

  /* at this point, the in-memory store knows about all the data that
     we have. next we will write the data out to disk.
     we need to invent some handles for the file store too. They can overlap
     with model handles, though here I make sure that they do not.
  */

  // damsel_store *filestore;
  unlink(FILENAME); // maybe it doesn't exist, so we'll just silently ignore any error
  // filestore = DMSLstore_create_hdf5(FILENAME);
  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "WRITE");
#ifdef DAMSEL_SUBFILE
  DMSLtrait_put(tb, "dmsl_num_subfiles", "2");
#endif
  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);

  DMSLtrait_release(tb);

  DMSLmodel_map_handles_inventing_file_handles(vertex_collection_id);

  /* we also need to map the handles for the tags which contain values
     that we want to store.
     This happens by creating a collection containing the tag to map, and
     mapping that into the file store. */
  damsel_handle tag_collection_vector[] = {data_tag_handle};

  damsel_container tag_collection_id;
  err = DMSLcontainer_create_vector(model, tag_collection_vector, /* count = */ 1, &tag_collection_id);

  DMSLmodel_map_handles_inventing_file_handles(tag_collection_id);

  /* Now we can tell damsel to execute everything. */

  // MPI_Request req;
  damsel_request_t req;
  // DMSLexecute(model, m_memstore->attached_file_store, &req);
  DMSLmodel_transfer_async(model, DAMSEL_TRANSFER_TYPE_WRITE, &req);
  // MPI_Status status;
  damsel_status_t status;
  //MPI_Wait(&req, &status);
  DMSLmodel_wait(req, &status);

  DMSLmodel_close(model);
  DMSLcontainer_close(tag_collection_id);
  DMSLcontainer_close(vertex_collection_id);
  DMSLlib_finalize(lib);

  MPI_Finalize();
  return 0;
}

