#include <pnetcdf.h>
#include <stdio.h>
#include <string.h>

#include "damsel.h"

static void handle_error(int status);

static void handle_error(int status) {
  fprintf(stderr, "%s\n", ncmpi_strerror(status));
}



int main(int argc, char **argv){
	MPI_Init(&argc, &argv);

	int rank, nprocs, ncfile, status;
        MPI_Comm_rank(MPI_COMM_WORLD, &rank);
        MPI_Comm_size(MPI_COMM_WORLD, &nprocs);

	/*Define stuff using Ben's example form pdf*/
	
	int data[]={3, 1, 4, 1, 5};
	/*initializing damsel library*/
	damsel_library lib;
	damsel_err_t err = DMSLlib_init(&lib);

	damsel_model model;
	err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
	damsel_memory_store *ms = (damsel_memory_store *)model;

	damsel_handle data_tag_h = 10203;
	DMSLtag_define(model, &data_tag_h, DAMSEL_DATA_TYPE_SHORTINT, "somedata");
	
	damsel_handle vertex_coll_h = 10020;
	damsel_handle base_handle = 20000;
	int step_size = 1000, quantity = 5;

	damsel_id_t vertex_coll_id;
        err = DMSLcontainer_create_sequence(model, vertex_coll_h, 20000, quantity, step_size, &vertex_coll_id);

	DMSLmodel_map_tag(data, vertex_coll_id, &data_tag_h);

	/*I'm skipping the part where it attaches to a specific model, need to develop a function for model_attach and corresponding hdf5 create functions*/
	/*and creates filestore handles*/


	printf("Writing pnetcdf file...\n");

	/*Parallel netCDF I/O part, once handle, sequences, etc been created, we are ready to dump the data in pnetcdf files. Each process writes the handles it generated. At this point we are making sure through the application that there are no overlaps.*/

	/*Define all variables*/
	int seq_handle_varid, vec_content_varid, tag_data_varid, ndims=1, dimid; 
	MPI_Offset nHandle = 5;
	
	status = ncmpi_create(MPI_COMM_WORLD, "output/damsel-pnetcdf-out.nc",
                NC_WRITE|NC_64BIT_DATA, MPI_INFO_NULL, &ncfile);
        if (status != NC_NOERR) handle_error(status);

	status = ncmpi_def_dim(ncfile, "num_elements_in_a_sequence", nHandle, &dimid); //change later
        if (status != NC_NOERR) handle_error(status);

	/*Saba: Variable definition for the damsel handle sequence -> a sequence of damsel handles, not sure should I use term collection here*/
	/*So I define a variable damsel handle sequence, and then I define attributes on them which will be base handle, quantity, and step size
 * 	not sure if this is a good approach but apparently it seems like a good idea*/

	//int base_handle = vertex_coll_h;
	status = ncmpi_def_var(ncfile, "damsel_handle_collection", NC_INT, 0, &dimid, &seq_handle_varid);
	if (status != NC_NOERR) handle_error(status);
	
	status = ncmpi_put_att_int(ncfile, seq_handle_varid, "handle", NC_INT, 1, &vertex_coll_h); //This attribute value is same as the seq_handle_varid i.e. value of the sequence handle 
        if(status != NC_NOERR) handle_error(status);

	status = ncmpi_put_att_int(ncfile, seq_handle_varid, "base_handle", NC_INT, 1, &base_handle);
        if(status != NC_NOERR) handle_error(status);

	status = ncmpi_put_att_int(ncfile, seq_handle_varid, "quantity", NC_INT, 1, &quantity);
        if(status != NC_NOERR) handle_error(status);

	status = ncmpi_put_att_int(ncfile, seq_handle_varid, "step", NC_INT, 1, &step_size); //no value at this point
        if(status != NC_NOERR) handle_error(status);

 	/*End of variable definition for the damsel handle sequence*/

	/*Variable definition for the damsel handle vectors -> a collection (use some other term) of vectors, it has a vector handle, contents and total number of the contents. I am thinking to define a variable for vector contents and define handle and quantity as attributes to it*/
	status = ncmpi_def_var(ncfile, "damsel_vector_collection", NC_INT, 1, &dimid, &vec_content_varid); //need to see what would be the type of it, it should be same type as damsel handles I think.
        if(status != NC_NOERR) handle_error(status);

	status = ncmpi_put_att_int(ncfile, vec_content_varid, "handle", NC_INT, 1, &vertex_coll_h); 
        if(status != NC_NOERR) handle_error(status);
	
	status = ncmpi_put_att_int(ncfile, vec_content_varid, "quantity", NC_INT, 1, &quantity);
        if(status != NC_NOERR) handle_error(status);

	/*End of variable definition*/      
 
	/*Now define variables for the tag definition and data*/
	/*In this version, I define damsel_tag as a variable, which will hold data and then name, handle, type and quantity as attributes of that tag*/

	char str_attribute[50] = "some_data_value_for_tag";
	int tag_handle = data_tag_h; 

	status = ncmpi_def_var(ncfile, "damsel_tag_data", NC_INT, 1, &dimid, &tag_data_varid);
        if(status != NC_NOERR) handle_error(status);

	status = ncmpi_put_att_int(ncfile, tag_data_varid, "tag_handle", NC_INT, 1, &tag_handle);
        if(status != NC_NOERR) handle_error(status);

	status = ncmpi_put_att_text(ncfile, tag_data_varid, "tag_name", strlen(str_attribute), str_attribute);
	if(status != NC_NOERR) handle_error(status);

	strcpy(str_attribute, "DENSE/SPARSE");
	status = ncmpi_put_att_text(ncfile, tag_data_varid, "tag_type", strlen(str_attribute), str_attribute);
        if(status != NC_NOERR) handle_error(status);

	status = ncmpi_put_att_int(ncfile, tag_data_varid, "tag_quantity", NC_INT, 1, &nHandle); //well dimid is number of handles so using it here for now
        if(status != NC_NOERR) handle_error(status);
	 /*End of variable definition*/

	ncmpi_enddef(ncfile);
	MPI_Offset start = 0, counter = 5; 
	/*Putting sequence collection data in the file*/	
	status = ncmpi_put_var_int_all(ncfile, seq_handle_varid, &vertex_coll_h);
        if (status != NC_NOERR) handle_error(status);

        status = ncmpi_put_vara_int_all(ncfile, tag_data_varid, &start, &counter, &data[0]);
	if (status != NC_NOERR) handle_error(status);

	status = ncmpi_close(ncfile);
	if (status != NC_NOERR) handle_error(status);

	//DMSLstore_close(model);
	//DMSLlib_finalize(lib);
	MPI_Finalize();
	return 0;
}
