#include "damsel.h"
#include "damsel-internal.h"
#include <mpi.h>

#include <assert.h>
#include <stdio.h>

#include "damsel-sugar.h"

damsel_handle handle_counter = 0xffff;

#define FILENAME "output/test-funnel.dmsl"
#define TAGNAME "funneled"

/* two processes write data to a damsel model */
void do_write(MPI_Comm comm, char *tagname)
{
    damsel_model model;
    damsel_trait_bundle tb;
    damsel_data thing;
    damsel_err_t err;

    int data, rank;

    err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);

    err = DMSLtrait_create(&tb);
    DMSLtrait_put(tb, "mode", "WRITE");
    DMSL_CHECK(DMSLmodel_attach(model, FILENAME, comm, tb));
    DMSLtrait_release(tb);

    MPI_Comm_rank(comm, &rank);
    data = 1000+rank;

    thing.tag_handle = handle_counter++;
    err = DMSLcontainer_create_sequence(model, handle_counter++, 1, 1, &thing.container);

    DMSL_CHECK(DMSLtag_define(model, &(thing.tag_handle), DAMSEL_DATA_TYPE_INTEGER,
		tagname));
    DMSL_CHECK(DMSLmodel_map_tag(&data, thing.container, &(thing.tag_handle)));

    DMSL_CHECK(DMSLmodel_map_handles_inventing_file_handles(thing.container));
    DMSL_CHECK(DMSLmodel_map_single_handle_inventing_file_handle(model, 
		thing.tag_handle));

    DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_WRITE);

    DMSL_CHECK(DMSLmodel_close(model));

}

void do_read(MPI_Comm comm, char *tagname)
{
    damsel_model model, fileside_model;
    damsel_model_internal *model_internal;
    damsel_trait_bundle tb;

    int *data, i, nitems;

    damsel_data mem, file;

    damsel_err_t err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);

    err = DMSLtrait_create(&tb);
    DMSLtrait_put(tb, "mode", "READ");
    DMSL_CHECK(DMSLmodel_attach(model, FILENAME, comm, tb));
    DMSLtrait_release(tb);
    DMSL_CHECK(DMSLmodel_populate(model));

    model_internal = (damsel_model_internal *)model;
    fileside_model = (damsel_model) model_internal->attached_file;

    err = DMSLselect_tag_by_name(model, tagname, &file.tag_handle);
    err = DMSLcontainer_create_sequence(fileside_model,
	    file.tag_handle, 1, 1, &file.tag_container);

    mem.tag_handle = handle_counter; handle_counter++;
    err = DMSLcontainer_create_sequence(model, 
	    mem.tag_handle, 1, 1, &mem.tag_container);

    DMSL_CHECK(DMSLmodel_map_handles(mem.tag_container, file.tag_container));

    file.container = DMSLselect_handles_with_values(model, file.tag_handle);
    nitems = DMSLcontainer_count(file.container);
    data = calloc(nitems, sizeof(int));

    mem.container_id = handle_counter; handle_counter += nitems;
    err = DMSLcontainer_create_sequence(model, mem.container_id,
	    nitems, 1, &mem.container);
    DMSL_CHECK(DMSLmodel_map_handles(mem.container, file.container));
    DMSL_CHECK(DMSLmodel_map_tag(data, mem.container, &(mem.tag_handle)));

    DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_READ);

    fprintf(stderr, "Got ");
    for (i=0; i<nitems; i++) {
	fprintf(stderr, "||%d", data[i]);
    }
    fprintf(stderr, "||\n");
    free(data);
    DMSL_CHECK(DMSLmodel_close(model));
}

int main(int argc, char **argv)
{
    int threadlvl, nprocs, rank;
    
    MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &threadlvl);
    assert(threadlvl == MPI_THREAD_MULTIPLE);
    damsel_library lib;
    damsel_err_t err = DMSLlib_init(&lib);

    MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    /*
    MPI_Comm newcomm;
    MPI_Comm_split(MPI_COMM_WORLD, ((rank<2) ? 1 : MPI_UNDEFINED), 0, &newcomm);
    */
    if (nprocs < 2) {
	fprintf(stderr, "run %s with 2 or more procs\n", argv[0]);
	return 0;
    }

    do_write(MPI_COMM_WORLD, TAGNAME);
    if (rank == 0) do_read(MPI_COMM_SELF, TAGNAME);

    DMSLlib_finalize(lib);
    MPI_Finalize();

    return 0;
}
