/* test get_contents for various types of containers */

#include <assert.h>
#include <stdio.h>
#include <mpi.h>
#include <unistd.h>

#include "damsel.h"

int main(int argc, char **argv) {
#ifndef DAMSEL_SINGLETHREAD
  int res;
#endif
  printf("%s\n", argv[0]);

#ifndef DAMSEL_SINGLETHREAD
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);

    /* test vector_get_contents */

  damsel_handle vector_of_handles[] = { 0x20000, 0x21000, 0x22000,
                                        0x23000, 0x24000 };
  damsel_container vertex_vec;
  err = DMSLcontainer_create_vector(model, vector_of_handles, 5, &vertex_vec);

  damsel_handle *handles = NULL;
  size_t hcount;
  err = DMSLcontainer_vector_get_contents(model, vertex_vec, &handles, &hcount);
  if (err.id != DMSL_OK.id || hcount != 5 || !handles) return 1;
  unsigned int i;
  for (i = 0; i < 5; i++) if (handles[i] != vector_of_handles[i]) return 1;

    /* test sequence_get_contents */
  damsel_handle start = 0x20000, start2, start3;
  size_t incr = 0x1000, count = 5, incr2, count2, incr3, count3;
  damsel_container vertex_seq;
  err = DMSLcontainer_create_sequence(model, start, count, incr, &vertex_seq);
  err = DMSLcontainer_sequence_get_contents(model, vertex_seq, &start2, &count2, &incr2);
  if (err.id != DMSL_OK.id || start2 != start || count2 != count || incr2 != incr) return 1;
  
    /* test tree_get_contents */
  start2 = 0x30000;
  count2 = 7, incr2 = 0x2000;
  damsel_container vertex_cont1;
  err = DMSLcontainer_create_sequence(model, start, count, incr, &vertex_cont1);
  damsel_container vertex_cont2;
  err = DMSLcontainer_create_sequence(model, start2, count2, incr2, &vertex_cont2);

  /* now wrap this inside a COLL container */
  damsel_container vertex_tree = DMSLcontainer_tree_create(model, DAMSEL_HANDLE_COLLECTION_TYPE_VECTOR);
  DMSLcontainer_tree_add(vertex_tree, vertex_cont1);
  DMSLcontainer_tree_add(vertex_tree, vertex_cont2);

  damsel_container cont;
  void *tree_node = NULL;
    // 1st call, should get vertex_cont1
  err = DMSLcontainer_tree_get_contents(model, vertex_tree, &tree_node, &cont);
  if (DMSL_OK.id != err.id || tree_node == NULL) return 1;
  
  err = DMSLcontainer_sequence_get_contents(model, cont, &start3, &count3, &incr3);
  if (DMSL_OK.id != err.id || start3 != start2 || count3 != count2 || incr3 != incr2) return 1;

    // 2nd call, should get vertex_cont2
  err = DMSLcontainer_tree_get_contents(model, vertex_tree, &tree_node, &cont);
  if (DMSL_OK.id != err.id || tree_node == NULL) return 1;
  
  err = DMSLcontainer_sequence_get_contents(model, cont, &start3, &count3, &incr3);
  if (DMSL_OK.id != err.id || start3 != start || count3 != count || incr3 != incr) return 1;

  DMSLmodel_close(model);
  DMSLcontainer_close(vertex_seq);
  DMSLcontainer_close(vertex_vec);
  DMSLcontainer_close(vertex_tree);
  DMSLcontainer_close(vertex_cont1);
  DMSLcontainer_close(vertex_cont2);

  DMSLlib_finalize(lib);

  return 0;
}
