
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <mpi.h>
#include <hdf5.h>
#ifndef HAVE_HDF5_MULTI_DATASET
# include "h5multidataset.h"
#endif

#include "damsel-ndc.h"

#define FILENAME "output/h5multidataset.h5"
#define NDSET		5
#define SIZE		10

int create_test_file();
int test_multi_write_read();

int main(int argc, char **argv)
{
    MPI_Init(&argc, &argv);
    // delete the old file
	unlink(FILENAME);
    create_test_file();
	test_multi_write_read();
	
    MPI_Finalize();
	return 0;
}

int create_test_file()
{
    int i;
    hid_t fid, sid, did;
	hsize_t dims[1] = {SIZE};
	char dsetname[16];
	
    fid = H5Fcreate (FILENAME, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
	sid = H5Screate_simple (1, dims, NULL);
	
	for (i=0; i<NDSET; i++) {
	    memset(dsetname, 0, 16);
		sprintf(dsetname, "dset%d", i);
	    did = H5Dcreate (fid, dsetname, H5T_STD_I32LE, sid, H5P_DEFAULT, 
		                 H5P_DEFAULT, H5P_DEFAULT);
        H5Dclose(did);
    }
	
	H5Sclose(sid);
	H5Fclose(fid);
	
	return 0;
}

int test_multi_write_read()
{
#ifdef DAMSEL_HDF5_MULTI
    int i, j;
    hid_t fid;
	char dsetname[16];
	int *rbuf, *wbuf;
    H5D_rw_multi_t dset_info[NDSET];

    hid_t plist_id = H5Pcreate(H5P_DATASET_XFER);
    H5Pset_dxpl_mpio(plist_id, H5FD_MPIO_COLLECTIVE);
    hid_t mpiprop_id = H5Pcreate(H5P_FILE_ACCESS);
    H5Pset_fapl_mpio(mpiprop_id, MPI_COMM_WORLD, MPI_INFO_NULL);
	

	rbuf = (int *)calloc(NDSET*SIZE, sizeof(int));
	wbuf = (int *)calloc(NDSET*SIZE, sizeof(int));
	
	for (i=0; i<NDSET; i++) {
		for (j=0; j<SIZE; j++)
	        wbuf[i*SIZE+j] = i*100+j;
	}
	
	fid = H5Fopen (FILENAME, H5F_ACC_RDWR, mpiprop_id);
	
	for (i=0; i<NDSET; i++) {
	    memset(dsetname, 0, 16);
		sprintf(dsetname, "dset%d", i);
		dset_info[i].dset_id = H5Dopen (fid, dsetname, H5P_DEFAULT);
        dset_info[i].mem_type_id = H5T_NATIVE_INT;
        dset_info[i].mem_space_id = H5S_ALL;
        dset_info[i].file_space_id = H5S_ALL;
        dset_info[i].wbuf = wbuf+i*SIZE;	
    }
	H5Dwrite_multi(fid, NDSET, dset_info, plist_id);

	// check if the data is correct from the file
    for (i=0; i<NDSET; i++)
	    dset_info[i].rbuf = rbuf+i*SIZE;
		
	H5Dread_multi(fid, NDSET, dset_info, plist_id);
	for (i=0; i<NDSET*SIZE; i++) {
	    if (wbuf[i] != rbuf[i]) {
			printf_ndc("H5Dread_multi() or H5Dwrite_multi() failed.");
		    return -1;
		}
	}
	
	/*
	for (i=0; i<NDSET; i++) {
	    printf("\n\n");
	    for (j=0; j<SIZE; j++) {
		    printf("%d\t", ((int *)(dset_info[i].buf))[j]);
			if ( ((j+1)%10)==0)
			    printf("\n");
		}
    }
	printf("\n");
	*/
	
	for (i=0; i<NDSET; i++)
	    H5Dclose(dset_info[i].dset_id);	

	H5Fclose(fid);
    H5Pclose(plist_id);
    H5Pclose(mpiprop_id);

    free(rbuf);
    free(wbuf);	
#endif
	
    return 0;
}
					
								  
