/*In this test case all processes define 5 handles each and write SolData with 5 values to these handles. 
 * In the file we can see SolData-"somevalue" data sets written by each process/rank. */

#include <assert.h>
#include "damsel-hdf5.h"
#include "damsel.h"
#include "damsel-internal.h"
#include "damsel-ndc.h"
#include "damsel-debug.h"
#include "damsel-connectivity.h"
#include "damsel-types.h"

#include <stdio.h>
#include <unistd.h>

#define FILENAME "output/nFilewrite.h5"


int main(int argc, char **argv) {  
  int i, len, res, myrank, num_procs;
#ifndef DAMSEL_SINGLETHREAD
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif
  MPI_Comm_rank (MPI_COMM_WORLD, &myrank); 
  MPI_Comm_size (MPI_COMM_WORLD, &num_procs); 

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  /* create a new model */
  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  assert(model != NULL);
  damsel_model_internal *ms = (damsel_model_internal *)model;
  assert(ms != NULL);

  short int data[5];
  len = 5;

  damsel_handle block_id [len]; 
  /*generating handles and some data values*/ 
  for(i=0; i<len; i++){
	block_id[i] = 100*myrank + i;
  	data[i] = 11*i + myrank;
  }

  /*All ranks will define the damsel handles, containers, and entity (5 each in this case)*/
  damsel_container block_collection_id;
  err = DMSLcontainer_create_vector(model, block_id, len, &block_collection_id); //assuming these are application handles?
  damsel_container block_handle_collection_id;
  err = DMSLcontainer_create_sequence(model, 200, len, 1, &block_handle_collection_id); //these are the handles for entities? Whats the difference between application handes and e.g. vertex handles in entity_define function. 
  DMSLentity_define(block_collection_id, DAMSEL_ENTITY_TYPE_VERTEX, 1, block_handle_collection_id);

  /*All ranks will define this tag handle with the same tag name.*/
  damsel_handle data_tag_handle = 1000;
  DMSLtag_define(model, &data_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "SolData");
  DMSLmodel_map_tag(data, block_collection_id, &data_tag_handle); 

  unlink(FILENAME); 
  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "WRITE");
#ifdef DAMSEL_SUBFILE
  DMSLtrait_put(tb, "dmsl_num_subfiles", "2");
#endif
  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);
 
  /*All ranks will call this inventing file handles as everyone has their own set of handles to which they are writing */
  DMSLmodel_map_handles_inventing_file_handles(block_collection_id);
  DMSLmodel_map_handles_inventing_file_handles(block_handle_collection_id);
  DMSLmodel_map_single_handle_inventing_file_handle(model, data_tag_handle);
  
  damsel_request_t req; 
  DMSLmodel_transfer_async(model, DAMSEL_TRANSFER_TYPE_WRITE, &req);
  damsel_status_t status;
  DMSLmodel_wait(req, &status);
  DMSLmodel_close(model);
  DMSLcontainer_close(block_collection_id);
  DMSLcontainer_close(block_handle_collection_id); 
  DMSLlib_finalize(lib);
  MPI_Finalize();
  return 0;
}

