/* test create_sequence and writing to disk */

#include <assert.h>
#include <stdio.h>
#include <unistd.h>

#include "mpi.h"
#include "damsel.h"

#define FILENAME "output/test-two-models.h5"
#define FILENAME2 "output/test-two-models-2.h5"

int main(int argc, char **argv) {
#ifndef DAMSEL_SINGLETHREAD
  int res;
#endif
  // some arbitrary data to write
  short int data[] = {3, 1, 4, 1, 5};
  short int data2[] = {1, 1, 2, 3, 5, 8, 13}; 

  printf("%s\n", argv[0]);

#ifndef DAMSEL_SINGLETHREAD
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model, model2;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  assert(model != NULL);

  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model2);
  assert(model2 != NULL);

  /* define a tag which will store our data. the tag will be
     named 'somedata' */

  damsel_handle data_tag_handle = 0x10203;
  DMSLtag_define(model, &data_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "somedata");

  /* we'll create a sequence handle collection with 5 handles in it. */

  damsel_container vertex_collection_id;
  err = DMSLcontainer_create_sequence(model,
    0x20000, 5, 0x1000, &vertex_collection_id);

  /* now set the values of the somedata tag on the handles */
  DMSLmodel_map_tag(data, vertex_collection_id, &data_tag_handle);

  /* at this point, the in-memory store knows about all the data that
     we have. next we will write the data out to disk.
     we need to invent some handles for the file store too. They can overlap
     with memstore handles, though here I make sure that they do not.
  */

  unlink(FILENAME); // maybe it doesn't exist, so we'll just silently ignore any error
  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "WRITE");

  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);

  DMSLmodel_map_handles_inventing_file_handles(vertex_collection_id);

  /* we also need to map the handles for the tags which contain values
     that we want to store.
     This happens by creating a collection containing the tag to map, and
     mapping that into the file store. */

  DMSLmodel_map_single_handle_inventing_file_handle(model, data_tag_handle);

  /* Now we can tell damsel to execute everything. */

  damsel_request_t req;
  DMSLmodel_transfer_async(model, DAMSEL_TRANSFER_TYPE_WRITE, &req);


  damsel_container vertex_collection_id2;
  err = DMSLcontainer_create_sequence(model2,
    0x20000, 5, 0x1000, &vertex_collection_id2);
  damsel_handle data_tag_handle2 = 0x10203;
  DMSLtag_define(model2, &data_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "somedata");

  DMSLmodel_map_tag(data2, vertex_collection_id2, &data_tag_handle2);

  /* at this point, the in-memory store knows about all the data that
     we have. next we will write the data out to disk.
     we need to invent some handles for the file store too. They can overlap
     with memstore handles, though here I make sure that they do not.
  */

  unlink(FILENAME2); // maybe it doesn't exist, so we'll just silently ignore any error

  DMSLmodel_attach(model2, FILENAME2, MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);

  DMSLmodel_map_handles_inventing_file_handles(vertex_collection_id2);

  /* we also need to map the handles for the tags which contain values
     that we want to store.
     This happens by creating a collection containing the tag to map, and
     mapping that into the file store. */

  DMSLmodel_map_single_handle_inventing_file_handle(model2, data_tag_handle2);

  /* Now we can tell damsel to execute everything. */

  damsel_request_t req2;
  DMSLmodel_transfer_async(model2, DAMSEL_TRANSFER_TYPE_WRITE, &req2);

  damsel_status_t status;
  DMSLmodel_wait(req, &status);

  damsel_status_t status2;
  DMSLmodel_wait(req2, &status2);

  DMSLmodel_close(model2);
  model2=NULL;

  DMSLmodel_close(model);
  model=NULL;

  DMSLcontainer_close(vertex_collection_id2);
  vertex_collection_id2=NULL;
  DMSLcontainer_close(vertex_collection_id);
  vertex_collection_id=NULL;

  DMSLlib_finalize(lib);
  lib=NULL;

  MPI_Finalize();
  return 0;
}

