
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#include <unistd.h>
#include <memory.h>

#include "damsel.h"
#include "damsel-internal.h"

int main(int argc, char **argv) {

  // we'll read the data into data[] and then check that
  // it matches up with target data check that
  // it matches up with target data
  short int targetdata[] = {3, 1, 4, 1, 5};
  short int data[5];

  printf("%s\n", argv[0]);
  if(argc!=2) {
    printf("specify filename of .h5 dump file to check\n");
    return 2;
  }

#ifndef DAMSEL_SINGLETHREAD
  int res;
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  assert(model != NULL);
  damsel_model_internal *ms = (damsel_model_internal *)model;
  assert(ms != NULL);

  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "READ");

  DMSLmodel_attach(model, argv[1], MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);

  DMSLmodel_populate(model);

  damsel_handle data_tag_handle = 0x31000;

  damsel_container tag_container_id;
  err = DMSLcontainer_create_sequence(model, 
    data_tag_handle, 1, 1, &tag_container_id);

  assert(tag_container_id != NULL);

  assert(model != NULL);
  damsel_handle fileside_data_tag_handle;
  err = DMSLselect_tag_by_name(model, "somedata", &fileside_data_tag_handle);
  damsel_container file_tag_container_id;
  err = DMSLcontainer_create_sequence((damsel_model)ms->attached_file, fileside_data_tag_handle, 1, 1, &file_tag_container_id);
  DMSLmodel_map_handles(tag_container_id, file_tag_container_id);

  // before we can read in collections, we need to read in containers, so that
  // they can be referred to in the collection structures.
  // this requirement should not stick around.

  int container_count;
  int tag_count;
  DMSLmodel_get_tuple_count(model, &container_count, &tag_count);
  assert(tag_count == 2);

  int collection_count;
  collection_count = DMSLmodel_get_collection_count(model);
  assert(collection_count == 1);

  damsel_collection_buf_type *collection_buf = malloc(collection_count * sizeof(damsel_collection_buf_type));

  DMSLmodel_get_collection_infos(model, collection_buf);

  assert(collection_buf[0].tag_count == 2);
  damsel_container filestore_vertex_container_id = collection_buf[0].contents;

  DMSLmodel_close_collection_infos(model, collection_buf);
  free(collection_buf);

  int flength = DMSLcontainer_count(filestore_vertex_container_id);
  assert(flength == 1);

  DMSLmodel_close(model);
  DMSLcontainer_close(tag_container_id);
  DMSLcontainer_close(file_tag_container_id);

  DMSLlib_finalize(lib);

  MPI_Finalize();
  return 0;
}

