/* test type_array_create and writing to disk */

#include <assert.h>
#include <stdio.h>
#include <unistd.h>

#include "mpi.h"
#include "damsel.h"

#define FILENAME "output/test-type-array.h5"


int main(int argc, char **argv) {
  short int data[][3] = {{3, 3, 3}, {1, 1, 1}, {4, 4, 4}, {1, 1, 1}, {5, 5, 5}}; // some arbitrary data to write

  printf("%s\n", argv[0]);

#ifndef DAMSEL_SINGLETHREAD
  int res;
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  assert(model != NULL);

  /* Create an array datatype to use for the data */
  size_t array_dim[1] = {3};
  damsel_data_type array_type = DMSLtype_array_create(DAMSEL_DATA_TYPE_SHORTINT, 1, array_dim);

  /* define a tag which will store our data. the tag will be
     named 'somedata' */

  damsel_handle data_tag_handle = 0x10203;
  DMSLtag_define(model, &data_tag_handle, array_type, "somedata");

  /* define a collection of the handles on which we will store data. we
     will store the data on handles 20000 .. upwards. */

  /* we'll create a sequence handle collection with 5 handles in it. */

  damsel_container vertex_collection_id;
  err = DMSLcontainer_create_sequence(model, 0x20000, 5, 0x1000, &vertex_collection_id);

  /* now set the values of the somedata tag on the handles */
  DMSLmodel_map_tag(data, vertex_collection_id, &data_tag_handle);

  /* at this point, the in-memory store knows about all the data that
     we have. next we will write the data out to disk.
     we need to invent some handles for the file store too. They can overlap
     with memstore handles, though here I make sure that they do not.
  */

  unlink(FILENAME); // maybe it doesn't exist, so we'll just silently ignore any error
  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "WRITE");

  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);

  DMSLmodel_map_handles_inventing_file_handles(vertex_collection_id);

  /* we also need to map the handles for the tags which contain values
     that we want to store.
     This happens by creating a collection containing the tag to map, and
     mapping that into the file store. */

  damsel_container tag_collection_id;
  err = DMSLcontainer_create_sequence(model,
    data_tag_handle, 1, 1, &tag_collection_id);

// //  damsel_handle file_tag_collection_handle = 0x910022;
// //  damsel_container file_tag_collection_id;
//   err = DMSLcontainer_create_sequence(ms->attached_file_store, file_tag_collection_handle, 1, 0x910020, 1, &file_tag_collection_id);

  // DMSLmodel_map_handles(tag_collection_id, file_tag_collection_id); // REPLACE
  DMSLmodel_map_handles_inventing_file_handles(tag_collection_id);

  /* Now we can tell damsel to execute everything. */

  damsel_request_t req;
  DMSLmodel_transfer_async(model, DAMSEL_TRANSFER_TYPE_WRITE, &req);
  damsel_status_t status;
  DMSLmodel_wait(req, &status);

  DMSLmodel_close(model);
  DMSLcontainer_close(vertex_collection_id);
  DMSLcontainer_close(tag_collection_id);

  DMSLlib_finalize(lib);

  MPI_Finalize();
  return 0;
}


