#!/bin/bash

[ -f .testconfig ] && . .testconfig
function runsucc {
echo -n "test: ($FUNCTOR) $@"
if [ -n "$DAMSELTESTVERBOSE" ] ; then
  $FUNCTOR $@ 2>&1
else
  $FUNCTOR $@ > test-result.tmp 2>&1
fi
RES=$?
echo -n " -> "
if [ "$RES" -eq "0" ] ; then
  echo passed.
else
  echo failed
if [ -n "$DAMSELTESTVERBOSE" ]; then
  echo results already shown
else
  cat test-result.tmp
fi
  echo unexpected failure for test: $@
  exit 1
fi
}

# runmpi is like runsucc but runs
# the commandline inside MPI
# on the specified number of
# processors:
#   runmpi 4 ./hello
function runmpi {
NR=$1
shift
echo -n "test mpi, $NR ranks: $@"
if [ -n "$DAMSELTESTVERBOSE" ] ; then
   $DAMSELMPIRUN $NR $@ 2>&1
else
   $DAMSELMPIRUN $NR $@ > test-result.tmp 2>&1
fi
RES=$?
echo -n " -> "
if [ "$RES" -eq "0" ] ; then
  echo passed.
else
  echo failed
if [ -n "$DAMSELTESTVERBOSE" ]; then
  echo results already shown
else
  cat test-result.tmp
fi
  echo unexpected failure for test: $@
  exit 1
fi
}


function buildsucc {
echo -n build: $@

if [ -n "$DAMSELTESTVERBOSE" ] ; then
  $@ 2>&1
else
  $@ > test-result.tmp 2>&1
fi
RES=$?
echo -n " -> "
if [ "$RES" -eq "0" ] ; then
  echo passed.
else
  echo failed
if [ -n "$DAMSELTESTVERBOSE" ] ; then
  echo results already shown
else
  cat test-result.tmp
fi
  echo unexpected failure for test: $@
  exit 1
fi
}

function runfail {
echo -n test: $@
$@ > test-result.tmp 2>&1
RES=$?
echo -n " -> "
if [ "$RES" -ne "0" ] ; then
  echo test returned error as expected
else
  echo unexpected success
  cat test-result.tmp
  echo unexpected success for test: $@
  exit 1
fi
}


function tests {

OUT_DIR=output
if [ ! -d "$OUT_DIR" ]; then
  echo "mkdir $OUT_DIR"
  mkdir -p $OUT_DIR
fi

runsucc ./test-linkedlist

runsucc ./test-create-sequence
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-create-sequence.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-create-sequence.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-create-sequence.h5"
runsucc ./test-read-pi-by-tag-distinct-containers "$OUT_DIR/test-create-sequence.h5"

runsucc ./test-create-vector
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-create-vector.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-create-vector.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-create-vector.h5"
runsucc ./test-read-pi-by-tag-distinct-containers "$OUT_DIR/test-create-vector.h5"

runsucc ./test-create-tree
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-create-tree.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-create-tree.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-create-tree.h5"
runsucc ./test-read-pi-by-tag-distinct-containers "$OUT_DIR/test-create-tree.h5"

runsucc ./test-create-collection
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-create-collection.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-create-collection.h5"
runsucc ./test-read-pi-by-collection "$OUT_DIR/test-create-collection.h5"

runsucc ./test-create-collection-multiple
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-create-collection-multiple.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-create-collection-multiple.h5"

runsucc ./test-create-vector2
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-create-vector2.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-create-vector2.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-create-vector2.h5"


runsucc ./test-two-sequences
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-two-sequences.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-two-sequences.h5"

runsucc ./test-two-tags-one-collection-write
runsucc ./test-two-tags-one-collection-read $OUT_DIR/test-two-tags-one-collection.h5

runsucc ./test-create-tree-addfast
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-create-tree-addfast.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-create-tree-addfast.h5"

runsucc ./test-connectivity
runsucc ./test-damsel-hdf5-info ./$OUT_DIR/test-connectivity.h5

runsucc ./test-connectivity-fast
runsucc ./test-damsel-hdf5-info ./$OUT_DIR/test-connectivity-fast.h5


runsucc ./test-type-array-create
runsucc ./test-type-array-read "$OUT_DIR/test-type-array.h5"

runsucc ./test-type-enum-create
runsucc ./test-type-vector-create

runsucc ./test-type-struct-create
runsucc ./test-type-struct-read

runsucc ./test-type-handle-write
runsucc ./test-type-handle-read "$OUT_DIR/test-type-handle-write.h5"


runsucc ./test-connectivity-write-pi
runsucc ./test-read-pi-by-connectivity "$OUT_DIR/test-connectivity-write-pi.h5"

runsucc ./test-connectivity-write-pi-ord
runsucc ./test-read-pi-by-connectivity "$OUT_DIR/test-connectivity-write-pi-ord.h5"

runsucc ./test-connectivity-write-cubes
runsucc ./test-connectivity-write-cubes-ord

runsucc ./test-connectivity-write-tags
runsucc ./test-damsel-hdf5-info ./$OUT_DIR/test-connectivity-write-tags.h5

runsucc ./test-create-sequence-distinct-containers
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-create-sequence-distinct-containers.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-create-sequence-distinct-containers.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-create-sequence-distinct-containers.h5"

runsucc ./test-create-vector-big

runsucc ./test-get-contents

runsucc ./test-create-disjoint-data
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-create-disjoint-data.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-create-disjoint-data.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-create-disjoint-data.h5"
# TODO check the results to see not only have I got the right data in the file but that I haven't created too much data...

runsucc ./test-global-map-1
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-global-map-1.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-global-map-1.h5"

runmpi 2 ./test-global-map-1
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-global-map-1.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-global-map-1.h5"

runmpi 6 ./test-global-map-1
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-global-map-1.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-global-map-1.h5"

runsucc ./test-global-map-2
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-global-map-2.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-global-map-2.h5"

runmpi 2 ./test-global-map-2
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-global-map-2.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-global-map-2.h5"

runmpi 5 ./test-global-map-2
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-global-map-2.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-global-map-2.h5"

runmpi 2 ./test-global-map-3
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-global-map-3.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-global-map-3.h5"

runmpi 5 ./test-global-map-3
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-global-map-3.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-global-map-3.h5"

# should behave the same both inside and outside of mpirun for 1 rank
# but we might as well check.

runsucc ./test-create-mpi
runsucc ./test-read-pi-by-collection "$OUT_DIR/test-create-mpi.h5"
rm "$OUT_DIR/test-create-mpi.h5"

runmpi 1 ./test-create-mpi
runsucc ./test-read-pi-by-collection "$OUT_DIR/test-create-mpi.h5"
rm "$OUT_DIR/test-create-mpi.h5"

runmpi 2 ./test-create-mpi
runsucc ./test-read-pi-by-collection "$OUT_DIR/test-create-mpi.h5"
rm "$OUT_DIR/test-create-mpi.h5"

runmpi 3 ./test-create-mpi
runsucc ./test-read-pi-by-collection "$OUT_DIR/test-create-mpi.h5"
rm "$OUT_DIR/test-create-mpi.h5"

runmpi 11 ./test-create-mpi
runsucc ./test-read-pi-by-collection "$OUT_DIR/test-create-mpi.h5"
rm "$OUT_DIR/test-create-mpi.h5"

runsucc ./test-vector-big-in-file
runsucc ./test-vector-big-in-file-read $OUT_DIR/test-vector-big-in-file.h5

runsucc ./test-two-models
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-two-models.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-two-models.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-two-models.h5"

runsucc ./test-trait

runsucc ./test-reopen
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-reopen.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-reopen.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-reopen.h5"

runsucc ./test-reopen2
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-reopen2.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-reopen2.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-reopen2.h5"

runsucc ./test-reopen-populate
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-reopen-populate.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-reopen-populate.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-reopen-populate.h5"

runsucc ./test-reopen-sametag
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-reopen-sametag.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-reopen-sametag.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-reopen-sametag.h5"

runsucc ./test-reopen-split-a
runsucc ./test-reopen-split-b
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-reopen-split.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-reopen-split.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-reopen-split.h5"

runsucc ./test-write-existing-fail

runsucc ./test-overlap-bug
runsucc ./test-damsel-hdf5-info $OUT_DIR/test-overlap-bug.h5
runsucc ./test-read-pi-by-tag "$OUT_DIR/test-overlap-bug.h5"
runsucc ./test-read-pi-transaction "$OUT_DIR/test-overlap-bug.h5"

runsucc ./test-provdep


runsucc ./proto1

# Run again, to see how things work when the output already exists
runsucc ./proto1

# heh. coverage tests told me we didn't do this one.
runsucc ./proto1 continue

# test writing/reading of full mesh - commented out for now 'cuz it won't work
# runsucc ./test-fullmesh

# various combinations of proto2 that have given me trouble when restarting

runsucc ./proto2d
runsucc ./proto2d continue

runmpi 2 ./proto2d
runmpi 2 ./proto2d continue

runmpi 2 ./proto2d
runmpi 3 ./proto2d continue

runmpi 2 ./proto2d
runsucc ./proto2d continue

runsucc ./test-flash

runsucc ./test-fail-double-tag-creation

rm -f $OUT_DIR/test-pnetcdf-textattr.dmsl
runsucc ./test-pnetcdf-textattr

runmpi 2 ./test-nconnectivitywrite 

runsucc ./test-subarray-read 

runmpi 4 ./test-subarray-read

runsucc ./test-threadlvl

runsucc ./test-pnetcdf ${OUT_DIR}/test-pnetcdf-1.dmsl

# TODO: figure out why the 4 processor version fails.
# TODO: the 2 proc test fails on NMI as well.  rats.
# runmpi 2 ./test-pnetcdf output/test-pnetcdf-2.dmsl
#runmpi 4 ./test-pnetcdf output/test-pnetcdf-4.dmsl

runcucc ./test-h5multidataset

# tests that should fail

echo The following tests should fail:

runsucc ./test-connectivity-write-pi-wrong
runfail ./test-read-pi-by-connectivity "$OUT_DIR/test-connectivity-write-pi-wrong.h5"

runfail ./test-fail-tag-nomap

echo all tests run ok
exit 0
}

# set FUNCTOR to run the non-MPI tests prefixed with a string - this can be
# used to run inside some other program for queueing or for memory leak
# testing.

# this is for valgrind on benc's hosts:
#   export FUNCTOR="valgrind --dsymutil=yes --error-exitcode=42 --leak-check=full"

# On surveyor.alcf.anl.gov, this setting can be used when running the test
# suite inside a cobalt job to run non-MPI jobs.
#   FUNCTOR="cobalt-mpirun -nofree -np 1 -verbose 2"
# (This doesn't work for MPI jobs, which need to be run with
# something like this: cobalt-mpirun -nofree -np $NR -verbose 2 $@
# which is not prefix... hopefully that will get figured out in a
# later patch)

if [ -z "$DAMSELMPIRUN" ] ; then
  # export DAMSELMPIRUN="cobalt-mpirun -nofree -verbose 2 -np"
  export DAMSELMPIRUN="mpirun -n"
fi

# set $DAMSELTESTVERBOSE to anything in order to have all the output of the
# tests sent to the console rather than # rather than suppressed on success

# set $DAMSELTESTNOBUILD to anything in order to suppress rebuilding
# of the code; for example, when running inside cobalt on BG/P where the
# tests have been built elsewhere

if [ -z "$DAMSELTESTNOBUILD" ]; then
  pushd .
  cd ..
  buildsucc make clean
  buildsucc make -j 4 all
  popd
  buildsucc make -j 4 all
  
fi
tests

